@@ -28,6 +28,10 @@
 
 #include "pager.h"
 
+/* even 16 is pretty darn dubious. */
+#define MAX_REASONABLE_ROWS 16
+#define DEFAULT_ROWS 1
+
 typedef struct {
 	GtkWidget *applet;
 
@@ -52,7 +56,9 @@ typedef struct {
 	WnckPagerDisplayMode display_mode;
 	gboolean display_all;
 	int size;
-  
+
+	/* gconf listeners id */
+	guint listeners [3];
 } PagerData;
 
 static void display_properties_dialog (BonoboUIComponent *uic,
@@ -131,8 +137,18 @@ applet_change_pixel_size (PanelApplet *applet,
 }
 
 static void
-destroy_pager(GtkWidget * widget, gpointer data)
+destroy_pager(GtkWidget * widget, PagerData *pager)
 {
+	GConfClient *client = gconf_client_get_default ();
+
+	gconf_client_notify_remove (client, pager->listeners[0]);
+	gconf_client_notify_remove (client, pager->listeners[1]);
+	gconf_client_notify_remove (client, pager->listeners[2]);
+
+	pager->listeners[0] = 0;
+	pager->listeners[1] = 0;
+	pager->listeners[2] = 0;
+
 }
 
 static const BonoboUIVerb pager_menu_verbs [] = {
@@ -148,13 +164,15 @@ num_rows_changed (GConfClient *client,
 		  GConfEntry  *entry,
 		  PagerData   *pager)
 {
-	int n_rows = 2; /* Default value */
+	int n_rows = DEFAULT_ROWS;
 	
 	if (entry->value != NULL &&
 	    entry->value->type == GCONF_VALUE_INT) {
 		n_rows = gconf_value_get_int (entry->value);
 	}
-	
+
+        n_rows = CLAMP (n_rows, 1, MAX_REASONABLE_ROWS);
+        
 	pager->n_rows = n_rows;
 	pager_update (pager);
 
@@ -229,7 +247,7 @@ setup_gconf (PagerData *pager)
 
 	key = panel_applet_gconf_get_full_key (PANEL_APPLET (pager->applet),
 					       "num_rows");
-	gconf_client_notify_add(client, key,
+	pager->listeners[0] = gconf_client_notify_add(client, key,
 				(GConfClientNotifyFunc)num_rows_changed,
 				pager,
 				NULL, NULL);
@@ -239,7 +257,7 @@ setup_gconf (PagerData *pager)
 
 	key = panel_applet_gconf_get_full_key (PANEL_APPLET (pager->applet),
 					       "display_workspace_names");
-	gconf_client_notify_add(client, key,
+	pager->listeners[1] = gconf_client_notify_add(client, key,
 				(GConfClientNotifyFunc)display_workspace_names_changed,
 				pager,
 				NULL, NULL);
@@ -248,7 +266,7 @@ setup_gconf (PagerData *pager)
 
 	key = panel_applet_gconf_get_full_key (PANEL_APPLET (pager->applet),
 					       "display_all_workspaces");
-	gconf_client_notify_add(client, key,
+	pager->listeners[2] = gconf_client_notify_add(client, key,
 				(GConfClientNotifyFunc)all_workspaces_changed,
 				pager,
 				NULL, NULL);
@@ -275,15 +293,21 @@ fill_pager_applet(PanelApplet *applet)
 	error = NULL;
 	pager->n_rows = panel_applet_gconf_get_int (applet, "num_rows", &error);
 	if (error) {
+                g_printerr (_("Error loading num_rows value for workspace switcher applet: %s\n"),
+                            error->message);
 		g_error_free (error);
-		pager->n_rows = 2; /* Default value */
+                /* leave current value */
 	}
 
+        pager->n_rows = CLAMP (pager->n_rows, 1, MAX_REASONABLE_ROWS);
+
 	error = NULL;
 	display_names = panel_applet_gconf_get_bool (applet, "display_workspace_names", &error);
 	if (error) {
+                g_printerr (_("Error loading display_workspace_names value for workspace switcher applet: %s\n"),
+                            error->message);
 		g_error_free (error);
-		display_names = FALSE; /* Default value */
+                /* leave current value */
 	}
 
 	if (display_names) {
@@ -295,8 +319,10 @@ fill_pager_applet(PanelApplet *applet)
 	error = NULL;
 	pager->display_all = panel_applet_gconf_get_bool (applet, "display_all_workspaces", &error);
 	if (error) {
+                g_printerr (_("Error loading display_all_workspaces value for workspace switcher applet: %s\n"),
+                            error->message);
 		g_error_free (error);
-		pager->display_all = TRUE; /* Default value */
+                /* leave current value */
 	}
 	
 	pager->size = panel_applet_get_size (applet);
@@ -460,9 +486,9 @@ update_workspaces_model (PagerData *pager)
 	int nr_ws, i;
 	WnckWorkspace *workspace;
 	GtkTreeIter iter;
-	
+
 	nr_ws = wnck_screen_get_workspace_count (pager->screen);
-	
+        
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_workspaces_spin), nr_ws);
 
 	gtk_list_store_clear (pager->workspaces_store);
@@ -488,6 +514,8 @@ workspace_created (WnckScreen    *screen,
 		   WnckWorkspace *space,
 		   PagerData     *pager)
 {
+        g_return_if_fail (WNCK_IS_SCREEN (screen));
+        
 	update_workspaces_model (pager);
 	g_signal_connect (G_OBJECT (space), "name_changed",
 			  (GCallback) workspace_renamed, pager);
@@ -498,6 +526,7 @@ workspace_destroyed (WnckScreen    *screen,
 		     WnckWorkspace *space,
 		     PagerData     *pager)
 {
+        g_return_if_fail (WNCK_IS_SCREEN (screen));
 	update_workspaces_model (pager);
 }
 
@@ -555,6 +584,14 @@ workspace_name_edited (GtkCellRendererText *cell_renderer_text,
 	g_slist_free (list);
 }
 
+static gboolean
+delete_event (GtkWidget *widget, gpointer data)
+{
+	gtk_widget_hide (widget);
+	return TRUE;
+}
+
+
 #define WID(s) glade_xml_get_widget (xml, s)
 
 static void
@@ -604,6 +641,9 @@ setup_dialog (GladeXML  *xml,
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_rows_spin), pager->n_rows);
 
+	g_signal_connect (pager->properties_dialog, "delete_event",
+			  G_CALLBACK (delete_event),
+			  pager);
 	
 	g_signal_connect_swapped (WID ("done_button"), "clicked",
 				  (GCallback) gtk_widget_hide, pager->properties_dialog);
@@ -613,10 +653,10 @@ setup_dialog (GladeXML  *xml,
 	g_signal_connect (G_OBJECT (pager->num_workspaces_spin), "value_changed",
 			  (GCallback) num_workspaces_value_changed, pager);
 	
-	g_signal_connect_swapped (pager->screen, "workspace_created",
-				  (GCallback) workspace_created, pager);
-	g_signal_connect_swapped (pager->screen, "workspace_destroyed",
-				  (GCallback) workspace_destroyed, pager);
+	g_signal_connect (pager->screen, "workspace_created",
+                          (GCallback) workspace_created, pager);
+	g_signal_connect (pager->screen, "workspace_destroyed",
+                          (GCallback) workspace_destroyed, pager);
 
 	pager->workspaces_store = gtk_list_store_new (1, G_TYPE_STRING, NULL);
 	update_workspaces_model (pager);
